#pragma once

#include <string>

namespace esphome {
namespace dashboard_import {

std::string get_package_import_url();
void set_package_import_url(std::string url);

}  // namespace dashboard_import
}  // namespace esphome
